
	std::vector<TopoDS_Face> s_Outline_Faces, s_Outline_TempFaces;
	std::vector<TopoDS_Wire> s_Outline_Wires, s_Outline_TempWires;
	std::vector<TopoDS_Edge> s_Outline_Edges;
	std::map<std::string, std::vector<TopoDS_Face> > s_TextMapSurfs;
	std::map<std::string, std::vector<TopoDS_Wire> > s_TextMapCurs;
	std::map<std::string, std::pair<double, double> > s_TextMapBounds;

	void TextOutline_ClearAllRecord()
	{
		s_Outline_TempWires.clear();
		s_Outline_TempFaces.clear();
		s_TextMapSurfs.clear();
		s_TextMapCurs.clear();
		s_TextMapBounds.clear();
	}
	void TextOutline_ClearRecordTempFaces()
	{
		s_Outline_TempWires.clear();
		s_Outline_TempFaces.clear();
	}
	void TextOutline_Record(char* str, int bClearTemp)
	{
		s_TextMapSurfs[str] = s_Outline_TempFaces;
		s_TextMapCurs[str] = s_Outline_TempWires;
		std::vector<TopoDS_Shape> shapes;
		Bnd_Box box;
		lsSolidGeom.Template_Convert(s_Outline_TempWires, shapes);
		lsSolidGeom.GetDBShapesBndBox(shapes, box);
		gp_Pnt maxP, minP;
		box.GetMaxPnt(maxP);
		box.GetMinPnt(minP);
		s_TextMapBounds[str] = std::make_pair(maxP.X()-minP.X(),	maxP.Y()-minP.Y());
		if(bClearTemp)
			TextOutline_ClearRecordTempFaces();
	}

	void TextOutline_ClearAll()
	{
		s_Outline_Faces.clear();
		s_Outline_Wires.clear();
		s_Outline_Edges.clear();
	}
	void TextOutline_Clear()
	{
		s_Outline_Wires.clear();
		s_Outline_Edges.clear();
	}
	void TextOutline_AddLine(int x1, int y1, int x2, int y2)
	{
		try
		{
			TopoDS_Edge edge = BRepBuilderAPI_MakeEdge(gp_Pnt(x1, y1, 0), gp_Pnt(x2, y2,0));
			s_Outline_Edges.push_back(edge);
		}
		catch(...)
		{
		}
	}
	void TextOutline_AddBezier(int x1, int y1, int x2, int y2, int x3, int y3)
	{
		try
		{
			TColgp_Array1OfPnt CurvePoles(1, 3);
			CurvePoles.SetValue(1, gp_Pnt(x1, y1,0));
			CurvePoles.SetValue(2, gp_Pnt(x2, y2,0));
			CurvePoles.SetValue(3, gp_Pnt(x3, y3,0));
			Handle_Geom_BezierCurve  hCur = new Geom_BezierCurve(CurvePoles);
			TopoDS_Edge edge = BRepBuilderAPI_MakeEdge(hCur);
			s_Outline_Edges.push_back(edge);
		}
		catch(...)
		{
		}
	}
	void TextOutline_AddWire()
	{
		std::vector<TopoDS_Shape> sortEdgeWires;
		std::vector<int> bClosedArray;
		if(lsSolidGeom.SortEdgesForConstructMultipleWire_OmitCloseTag(s_Outline_Edges, sortEdgeWires, bClosedArray))
		{
			for(int i=0; i<sortEdgeWires.size(); ++i)
			{
				if(sortEdgeWires[i].ShapeType()==TopAbs_WIRE)
				{
					s_Outline_Wires.push_back(TopoDS::Wire(sortEdgeWires[i]));
					s_Outline_TempWires.push_back(s_Outline_Wires.back());
				}
			}
		}
		if(0)
			lsSolidGeom.AddTempShapeToDB(s_Outline_Edges);
		s_Outline_Edges.clear();
	}
	void TextOutline_MakeFace()
	{
		std::vector<TopoDS_Face> faces;
		if(0)
			lsSolidGeom.AddTempShapeToDB(s_Outline_Wires);
		try
		{
			if(lsSolidGeom.Aux_FillPlaneByWires(s_Outline_Wires, faces))
			{
				s_Outline_Faces.insert(s_Outline_Faces.end(), faces.begin(), faces.end());
				s_Outline_TempFaces.insert(s_Outline_TempFaces.end(), faces.begin(), faces.end());
			}
		}
		catch(...)
		{
		}
		s_Outline_Wires.clear();
	}

	int TextOutline_AddToDB(int type)
	{
		try
		{
			if(s_Outline_Faces.empty())
				return 0;
			std::vector<TopoDS_Shape> addShapes, delShapes;
			for(int i=0; i<s_Outline_Faces.size(); ++i)
			{
				addShapes.push_back(s_Outline_Faces[i]);
			}
			if(!lsSolidGeom.AddAndRemoveEntities(addShapes, delShapes))
				return 0;
		}
		catch(...)
		{
			return 0;
		}
		return 1;
	}
	void TextOutline_PreviewFace()
	{
		for(int i=0; i<s_Outline_TempFaces.size(); ++i)
			lsSolidGeom.Aux_PreviewFace(s_Outline_TempFaces[i]);
	}
	void TextOutline_PreviewSolid(double zExt)
	{
		gp_Vec vec(0,0,zExt);
		TopoDS_Shape shape;
		for(int i=0; i<s_Outline_TempFaces.size(); ++i)
		{
			shape= BRepPrimAPI_MakePrism(s_Outline_TempFaces[i], vec, Standard_True);
			lsSolidGeom.Aux_PreviewFaces(shape);
		}
	}
	void TextOutline_PreviewWires()
	{
		for(int i=0; i<s_Outline_TempWires.size(); ++i)
			lsSolidGeom.Aux_PreviewEdges(s_Outline_TempWires[i]);
	}
	int TextOutline_ModifyXExt(char* str, double xExt, double* newYExt)
	{
		std::map<std::string, std::pair<double, double> >::iterator it;
		if((it=s_TextMapBounds.find(str))!=s_TextMapBounds.end())
		{
			*newYExt = xExt/it->second.first*it->second.second;
			return 1;
		}
		return 0;
	}
	int TextOutline_ModifyYExt(char* str, double yExt, double* newXExt)
	{
		std::map<std::string, std::pair<double, double> >::iterator it;
		if((it=s_TextMapBounds.find(str))!=s_TextMapBounds.end())
		{
			*newXExt = yExt/it->second.second*it->second.first;
			return 1;
		}
		return 0;
	}
	int TextOutline_PreviewRecord(char* str, int type, double xExt, double yExt, double zExt, double meshSize)
	{
		std::map<std::string, std::vector<TopoDS_Wire> >::iterator it;
		std::map<std::string, std::vector<TopoDS_Face> >::iterator it2;
		int i;
		if(type==0)
		{
			if((it=s_TextMapCurs.find(str))!=s_TextMapCurs.end())
			{
				s_Outline_TempWires = it->second;
				std::pair<double, double> s = s_TextMapBounds[str];
				double scale = xExt/s.first;
				gp_Trsf transfer;
				transfer.SetScale(gp_Pnt(0,0,0), scale);
				for(i=0; i<s_Outline_TempWires.size(); ++i)
				{
					BRepBuilderAPI_Transform brepTransform(s_Outline_TempWires[i], transfer, true);
					s_Outline_TempWires[i] = TopoDS::Wire(brepTransform.Shape());
				}
				TextOutline_PreviewWires();
				return 1;
			}
		}
		else if(type==1||type==2)
		{
			if((it2=s_TextMapSurfs.find(str))!=s_TextMapSurfs.end())
			{
				s_Outline_TempFaces = it2->second;
				std::pair<double, double> s = s_TextMapBounds[str];
				double scale = xExt/s.first;
				gp_Trsf transfer;
				transfer.SetScale(gp_Pnt(0,0,0), scale);
				for(i=0; i<s_Outline_TempFaces.size(); ++i)
				{
					BRepBuilderAPI_Transform brepTransform(s_Outline_TempFaces[i], transfer, true);
					s_Outline_TempFaces[i] = TopoDS::Face(brepTransform.Shape());
				}
				if(type==1)
					TextOutline_PreviewFace();
				else if(type==2)
					TextOutline_PreviewSolid(zExt);
				return 1;
			}
		}
		return 0;
	}	
	int TextOutline_AddRecord(char* str, int type, double xExt, double yExt, double zExt, double meshSize)
	{
		std::map<std::string, std::vector<TopoDS_Wire> >::iterator it;
		std::map<std::string, std::vector<TopoDS_Face> >::iterator it2;
		std::vector<TopoDS_Shape> addShapes, delShapes;
		if(type==0)
		{
			if((it=s_TextMapCurs.find(str))!=s_TextMapCurs.end())
			{
				s_Outline_TempWires = it->second;
				std::pair<double, double> s = s_TextMapBounds[str];
				double scale = xExt/s.first;
				gp_Trsf transfer;
				transfer.SetScale(gp_Pnt(0,0,0), scale);
				for(int i=0; i<s_Outline_TempWires.size(); ++i)
				{
					BRepBuilderAPI_Transform brepTransform(s_Outline_TempWires[i], transfer, true);
					s_Outline_TempWires[i] = TopoDS::Wire(brepTransform.Shape());
					addShapes.push_back(s_Outline_TempWires[i]);
				}
				if(!lsSolidGeom.AddAndRemoveEntities(addShapes, delShapes))
					return 0;
				return 1;
			}
		}
		else if(type==1)
		{
			if((it2=s_TextMapSurfs.find(str))!=s_TextMapSurfs.end())
			{
				s_Outline_TempFaces = it2->second;
				std::pair<double, double> s = s_TextMapBounds[str];
				double scale = xExt/s.first;
				gp_Trsf transfer;
				transfer.SetScale(gp_Pnt(0,0,0), scale);
				for(int i=0; i<s_Outline_TempFaces.size(); ++i)
				{
					BRepBuilderAPI_Transform brepTransform(s_Outline_TempFaces[i], transfer, true);
					s_Outline_TempFaces[i] = TopoDS::Face(brepTransform.Shape());
					addShapes.push_back(s_Outline_TempFaces[i]);
				}
				if(!lsSolidGeom.AddAndRemoveEntities(addShapes, delShapes))
					return 0;
				return 1;
			}
		}
		else if(type==2)
		{
			if((it2=s_TextMapSurfs.find(str))!=s_TextMapSurfs.end())
			{
				s_Outline_TempFaces = it2->second;
				gp_Vec vec(0,0,zExt);
				TopoDS_Shape shape;
				std::pair<double, double> s = s_TextMapBounds[str];
				double scale = xExt/s.first;
				gp_Trsf transfer;
				transfer.SetScale(gp_Pnt(0,0,0), scale);
				for(int i=0; i<s_Outline_TempFaces.size(); ++i)
				{
					BRepBuilderAPI_Transform brepTransform(s_Outline_TempFaces[i], transfer, true);
					s_Outline_TempFaces[i] = TopoDS::Face(brepTransform.Shape());
					shape= BRepPrimAPI_MakePrism(s_Outline_TempFaces[i], vec, Standard_True);
					addShapes.push_back(shape);
				}
				if(!lsSolidGeom.AddAndRemoveEntities(addShapes, delShapes))
					return 0;
				return 1;
			}
		}
		return 0;
	}	